/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2012 UJF-Grenoble 1, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/
// include from STD
#include <set>

// includes from Qt
#include <QMap>
#include <QDir>
#include <QPair>
#include <QString>
#include <QFileInfo>
#include <QTextStream>

#include <Parameters.hxx>

#include <Action.hxx>

#ifndef ACTIONGENERATOR_H
#define ACTIONGENERATOR_H

using namespace coreschema;

class ActionGenerator {
public :
	static bool generateActionFiles(QString xmlFileName, QString devDirectoryName, QString * elementClassName = NULL);
		
protected:
	ActionGenerator(QString xmlFilename, QString devDirectoryName);

    virtual void setXmlFileName(QString xmlFileName) throw (QString);
    virtual void setDevDirectoryName(QString devDirectoryName) throw (QString);


    virtual void createElement() throw (QString);
	virtual void generateActionFiles() throw (QString);

	virtual void writeHFile() throw (QString);
	virtual void writeCFile() throw (QString);
	virtual void writeImplementationFile() throw (QString);

	/** XML DOM Element create by CodeSynthesis
	 * This action contains all of the informations of the xml file
	 */
    std::auto_ptr<Action> theAction;

	/// Where to find the XML description of the Action
	QFileInfo xmlFileName;

	/// Where to store the produced files
    QDir devDirectoryName;

	/// Name of the created Action class 
	///  redundant information already contained in theAction, for practicle purpose.
    QString className;
	
	/// Does the action have parameters stored as properties ?
	bool hasParameters;
	/** List containing all parameters of the action stored as properties of theAction
	 *  For practical reasons, this map contains for each parameter:
	 *  - key: the name of the parameter
	 *  - value: a QPair of element: 
	 *       - first: the type of the parameter
	 *       - second: the default value of the parameter 
	 *                 (if non exists in the XML file, a default one is given) 
	 */
	QMap<QString, QPair<QString, QString> > parameters;

	/// Set of additional files to include
	std::set<QString> additionalIncludes;

private:
	bool isItkFilter;
//    bool hasImplementationFile;
};

#endif
